//Authors: Ates Akaydn, M.Bugra Yildiz
// Phong Globe Shader
// Date: 26.10.2011

//#version 140

//Input
//TODO: Support for multiple lights will be added in the near future
uniform vec4 lightColor; //Color of the scene light
uniform vec4 lightPosition; //Position of the scene light

//TODO: Light attenuation parameters will be controllable from the application in the near future
uniform float lightAttRange = float(250.0); //Range of light attenuation... where it perishes
uniform float lightAttExp = float(2.0); //Light attenuation exponent... controls how rapidly light intensity drops

//Texture Parameters
uniform sampler2D earthTexture; //Texture of earh
uniform float earthTextureAlpha = float(1.0); //Earth texture transparency control
uniform sampler2D atmosphereTexture; //Texture of atmosphere 
uniform float atmosphereBlend = float(1.0); //atmosphere texture blending control
uniform sampler2D decalTexture; //Decal Texture
uniform float decalBlend = float(1.0); //Decal texture blending control
uniform samplerCube cubeMap; //Cube Map texture
uniform float cubeMapBlend = float(1.0); //Decal texture blending control
uniform sampler2D specularTexture; //Specular Map
uniform sampler2D ambientTexture; //Ambient Map

//Material Parameters
uniform vec4 matAmbientCoef = vec4( 0.01, 0.01, 0.01, 1.0 );
uniform vec4 matDiffuseCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform vec4 matSpecularCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform float matShininess = 8.0; 

//Interpolated parameters from vertex shader
varying vec3 iPos, iNormal, iEyeDir, iReflectDir, iLightDir, iTexCoord; //Interpolated output normal, eye direction and light direction vectors

void main()
{

	vec4 earthTexColor = texture(earthTexture, iTexCoord.xy);
	matDiffuseCoef.a = earthTextureAlpha;
	vec4 atmosphereTexColor = texture(atmosphereTexture, iTexCoord.xy);	
	vec4 cubeMapTexColor = textureCube( cubeMap, iReflectDir );
	
	/////////////////////////////////////
	// DECAL EXAMPLE - Via Decall Texture
	vec4 decalTexColor = texture(decalTexture, iTexCoord.xy);
	vec3 decalRGB = mix( earthTexColor.rgb, decalTexColor.rgb, decalTexColor.a * decalBlend);
	vec4 decalColor = vec4( decalRGB, earthTexColor.a );
	/////////////////////////////////////
	
	/////////////////////////////////////
	// BLEND EXAMPLE - Via atmosphere Texture -
	vec3 blendedRGB = mix( decalColor.rgb, atmosphereTexColor.rgb, atmosphereTexColor.rgb * atmosphereBlend);
	vec4 blendedColor = vec4(blendedRGB, decalColor.a * atmosphereTexColor.a );
	/////////////////////////////////////
	
	/////////////////////////////////////
	// BLEND EXAMPLE - Via cubemap Texture -
	vec3 envRGB = mix( blendedColor.rgb, cubeMapTexColor.rgb, cubeMapTexColor.rgb * cubeMapBlend);
	vec4 envColor = vec4( envRGB, blendedColor.a * cubeMapTexColor.a );	
	/////////////////////////////////////	

	vec4 ambientTexColor = texture(ambientTexture, iTexCoord.xy);	
	vec4 diffuseTexColor = envColor;	
	vec4 specularTexColor = texture(specularTexture, iTexCoord.xy);	
	
	vec4 final_color = matAmbientCoef * ambientTexColor;
	vec3 N = normalize(iNormal);
    vec3 L = normalize(iLightDir);
    
    float lambertTerm = dot(N,L);
    if (lambertTerm > 0.0)
    {
		float r = length( lightPosition.xyz - iPos );
		float k = 1 / pow(lightAttRange,2.0);
		float I = length(lightColor);
		
		float lightAtt = clamp( I / (1 + k * pow( r, lightAttExp ) ),0.0,1.0 );
		
		final_color.rgb += (lightColor.rgb * matDiffuseCoef.rgb * diffuseTexColor.rgb *lambertTerm) * lightAtt;
		vec3 E = normalize(iEyeDir);
		vec3 R = reflect(-L, N);
		float specular = pow(max(dot(R, E), 0.0), matShininess);
		final_color.rgb += (lightColor.rgb * matSpecularCoef.rgb * specularTexColor.rgb *specular) * lightAtt;
    }
  
	final_color.a = matDiffuseCoef.a;
	gl_FragColor = final_color;	

}