//Authors: Ates Akaydn, M.Bugra Yildiz
// Phong Shader with Normal Mapping
// Date: 26.11.2011

//#version 140

//INPUT

//Texture Parameters
uniform sampler2D diffuseTexture; //Diffuse Texture
uniform sampler2D specularTexture; //Specular Map
uniform sampler2D ambientTexture; //Ambient Map
uniform sampler2D normalTexture; //Normal Map
uniform float tiling = float(1.0); //Amount of Tiling for all textures

//Light Properties
//TODO: Support for multiple lights will be added in the near future
uniform vec4 lightColor; //Color of the scene light
uniform vec4 lightPosition; //Position of the scene light

//TODO: Light attenuation parameters will be controllable from the application in the near future
uniform float lightAttRange = float(250.0); //Range of light attenuation... where it perishes
uniform float lightAttExp = float(2.0); //Light attenuation exponent... controls how rapidly light intensity drops

//Material Parameters
uniform vec4 matAmbientCoef = vec4( 0.01, 0.01, 0.01, 1.0 );
uniform vec4 matDiffuseCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform vec4 matSpecularCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform float matShininess = 8.0; 

//Interpolated parameters from vertex shader
varying vec3 iPos, iNormal, iEyeDir, iLightDir, iTexCoord; //Interpolated output normal, eye direction and light direction vectors

//Routine to approximate tangent vector from normal
//Taken from http://www.ozone3d.net/tutorials/mesh_deformer_p2.php#tangent_space
//TODO: Best way of doing it is to pass tangent vector in addition to normal vector
//From the application. Though current implementation does not enable it. Will be fixed soon...
vec3 approximateTangent( vec3 normal ){
	vec3 tangent;
	vec3 c1 = cross(normal, vec3(0.0, 0.0, 1.0)); 
	vec3 c2 = cross(normal, vec3(0.0, 1.0, 0.0)); 
	
	if(length(c1)>length(c2))
	{
		tangent = c1;	
	}
	else
	{
		tangent = c2;	
	}
	
	tangent = normalize(tangent);
	return tangent;
}

//Main routine
void main()
{
	vec4 ambientTexColor = texture(ambientTexture, tiling * iTexCoord.xy);
	vec4 diffuseTexColor = texture(diffuseTexture, tiling * iTexCoord.xy);
	vec4 specularTexColor = texture(specularTexture, tiling * iTexCoord.xy);
	vec3 normalTexColor = normalize( texture(normalTexture, tiling * iTexCoord.xy).xyz * 2.0 - 1.0 );
		
	vec4 final_color = matAmbientCoef * ambientTexColor;
	
	//TBN Matrix
	vec3 n = normalize(iNormal);
	vec3 t = approximateTangent(n); //tangent component of the TBN matrix
	vec3 b = cross( n,t ); //binormal component of the TBN matrix
	
	vec3 N = normalTexColor;
	
	vec3 tempDir = normalize(iLightDir);
    vec3 L = vec3( 0.0,0.0,0.0 );
	L.x = dot( tempDir, t );
	L.y = dot( tempDir, b );
	L.z = dot( tempDir, n );     

    float lambertTerm = dot(N,L);
    if (lambertTerm > 0.0)
    {
		float r = length( lightPosition.xyz - iPos );
		float k = 1 / pow(lightAttRange,2.0);
		float I = length(lightColor);
		
		float lightAtt = clamp( I / (1 + k * pow( r, lightAttExp ) ),0.0,1.0 );
		    
		final_color.rgb += (lightColor.rgb * matDiffuseCoef.rgb * diffuseTexColor.rgb *lambertTerm ) * lightAtt;
		tempDir = normalize(iEyeDir); 
		
		vec3 E = vec3(0.0,0.0,0.0);
		E.x = dot( tempDir, t );
		E.y = dot( tempDir, b );
		E.z = dot( tempDir, n ); 
		
		vec3 R = reflect(-L, N);
		float specular = pow(max(dot(R, E), 0.0), matShininess);
		final_color.rgb += (lightColor.rgb * matSpecularCoef.rgb * specularTexColor.rgb * specular) * lightAtt;
    }
	final_color.a = matDiffuseCoef.a;  
	gl_FragColor = clamp(final_color, 0.0, 1.0);	

}