//Authors: Ates Akaydn, M.Bugra Yildiz
// Phong Shader with Normal Mapping
// Date: 26.11.2011

//#version 140

//INPUT
const float pi = 3.14159; //Approximation for pi
uniform float time; //Current time
uniform float speed1 = float(6.4), wavelength1 = float(32.0), amplitude1 = float(0.7);
uniform float speed2 = float(9.6), wavelength2 = float(56.0), amplitude2 = float(0.8);
uniform float speed3 = float(4.0), wavelength3 = float(36.0), amplitude3 = float(0.9);
uniform vec2 wavedir1 = vec2(0.5,0.5), wavedir2 = vec2(1.0,0.0), wavedir3 = vec2(0.0,-1.0); 

//Texture Parameters
uniform sampler2D diffuseTexture; //Diffuse Texture
uniform sampler2D specularTexture; //Specular Map
uniform sampler2D ambientTexture; //Ambient Map
uniform samplerCube cubeMap; //Cube Map texture
uniform float cubeMapBlend = float(1.0); //Cubemap texture blending amount
uniform float tiling = float(1.0); //Amount of Tiling for all textures

//Light Properties
//TODO: Support for multiple lights will be added in the near future
uniform vec4 lightColor; //Color of the scene light
uniform vec4 lightPosition; //Position of the scene light

//TODO: Light attenuation parameters will be controllable from the application in the near future
uniform float lightAttRange = float(250.0); //Range of light attenuation... where it perishes
uniform float lightAttExp = float(2.0); //Light attenuation exponent... controls how rapidly light intensity drops

//Material Parameters
uniform vec4 matAmbientCoef = vec4( 0.01, 0.01, 0.01, 1.0 );
uniform vec4 matDiffuseCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform vec4 matSpecularCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform float matShininess = 8.0; 

//Interpolated parameters from vertex shader
varying vec3 iPos, wEyeDir, iEyeDir, iLightDir, iTexCoord; //Interpolated output normal, eye direction and light direction vectors

float dWavedx(float speed, float wavelength, float amplitude, vec2 direction, float x, float y) {
    float frequency = 2*pi/wavelength;
    float phase = speed * frequency;
    float theta = dot(direction, vec2(x, y));
    float A = amplitude * direction.x * frequency;
    return A * cos(theta * frequency + time * phase);
}

float dWavedy(float speed, float wavelength, float amplitude, vec2 direction, float x, float y) {
    float frequency = 2*pi/wavelength;
    float phase = speed * frequency;
    float theta = dot(direction, vec2(x, y));
    float A = amplitude * direction.y * frequency;
    return A * cos(theta * frequency + time * phase);
}

vec3 waveNormal(float x, float y) {
    float dx = 0.0;
    float dy = 0.0;
    
    dx += dWavedx(speed1, wavelength1, amplitude1, wavedir1, x, y);
    dx += dWavedx(speed2, wavelength2, amplitude2, wavedir2, x, y);
    dx += dWavedx(speed3, wavelength3, amplitude3, wavedir3, x, y);
    
    dy += dWavedy(speed1, wavelength1, amplitude1, wavedir1, x, y);
    dy += dWavedy(speed2, wavelength2, amplitude2, wavedir2, x, y);
    dy += dWavedy(speed3, wavelength3, amplitude3, wavedir3, x, y);
    
    vec3 n = vec3(-dx, 1.0, -dy);
    return normalize(n);
}

//Main routine
void main()
{
	vec4 ambientTexColor = texture(ambientTexture, tiling * iTexCoord.xy);
	vec4 diffuseTexColor = texture(diffuseTexture, tiling * iTexCoord.xy);
	vec4 specularTexColor = texture(specularTexture, tiling * iTexCoord.xy);
	
	vec3 wNormal = waveNormal( iPos.x, iPos.z );
	vec3 wReflectDir = reflect(-wEyeDir, wNormal);
	
	//This is view space direction... cube map exists in world space
	
	vec4 cubeMapTexColor = textureCube( cubeMap, wReflectDir );	
	

	//////////////////////////////////////////////////////
	//BLEND CUBEMAP TEXTURE TO THE FINAL COLOR			//
	vec3 envRGB = mix( diffuseTexColor.rgb, cubeMapTexColor.rgb, cubeMapTexColor.rgb * cubeMapBlend);
	vec4 envColor = vec4( envRGB, diffuseTexColor.a * cubeMapTexColor.a );
	diffuseTexColor = envColor;
	//////////////////////////////////////////////////////   		
		
	vec4 final_color = matAmbientCoef * ambientTexColor;
	
	vec3 iNormal = gl_NormalMatrix * wNormal;	
	vec3 N = normalize(iNormal);
	
	vec3 L = normalize(iLightDir);

    float lambertTerm = dot(N,L);
    if (lambertTerm > 0.0)
    {
		float r = length( lightPosition.xyz - iPos );
		float k = 1 / pow(lightAttRange,2.0);
		float I = length(lightColor);
		
		float lightAtt = clamp( I / (1 + k * pow( r, lightAttExp ) ),0.0,1.0 );
		    
		final_color.rgb += (lightColor.rgb * matDiffuseCoef.rgb * diffuseTexColor.rgb *lambertTerm ) * lightAtt;
		
		vec3 E = normalize(iEyeDir);
		
		vec3 R = reflect(-L, N);
		float specular = pow(max(dot(R, E), 0.0), matShininess);
		final_color.rgb += (lightColor.rgb * matSpecularCoef.rgb * specularTexColor.rgb * specular) * lightAtt;
    }
    final_color.a = matDiffuseCoef.a;
	gl_FragColor = clamp(final_color, 0.0, 1.0);	

}