//Authors: Ates Akaydn, M.Bugra Yildiz
// Teapot Shader
// Date: 26.11.2011

//#version 140

//Input
in vec4 vPosition; //Position of the vertex
in vec4 vNormal; //Normal vector of the vertex
in vec4 vTexCoord; //Texture coordinates of the vertex

uniform sampler2D displacementTexture; //Displacement Map

//Light Properties
//TODO: Support for multiple lights will be added in the near future
uniform vec4 lightColor; //Color of the scene light
uniform vec4 lightPosition; //Position of the scene light

//TODO: Light attenuation parameters will be controllable from the application in the near future
uniform float lightAttRange = float(250.0); //Range of light attenuation... where it perishes
uniform float lightAttExp = float(2.0); //Light attenuation exponent... controls how rapidly light intensity drops

//Material Parameters
uniform vec4 matAmbientCoef = vec4( 0.01, 0.01, 0.01, 1.0 );
uniform vec4 matDiffuseCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform vec4 matSpecularCoef = vec4( 1.0, 1.0, 1.0, 1.0 );
uniform float matShininess = 8.0; 

const vec3 teapotBasePos = vec3( -39.675, -35.0, 37.516 );
uniform float stretchFactor = 2;
const float dispFactor = 6;
const float dispOffset = 0.8;

//Output
varying vec3 iAmbientColor, iSpecularColor, iTexCoord; //Interpolated output normal, eye direction and light direction vectors
varying vec4 iDiffuseColor;

void main()
{	
	//Inverse transform normals. And carry out perspective correct interpolationg for fragment shader
	vec3 iNormal = gl_NormalMatrix * vNormal.xyz;
	iTexCoord = vTexCoord.xyz;
	vec3 iPos = vPosition.xyz;
	
	float squashFactor = 1/stretchFactor;
	vec3 stretchVector = vec3( squashFactor, stretchFactor, squashFactor ); 
	
	vec3 dispTexColor = texture(displacementTexture, vTexCoord.xy);
	float dispAmount = length(dispTexColor);
	
	vPosition.xyz = vPosition.xyz - teapotBasePos.xyz;
	vPosition.xyz = vPosition.xyz * stretchVector;
	vPosition.xyz = vPosition.xyz + (vNormal.xyz * dispAmount * dispFactor * (stretchFactor - dispOffset));
	vPosition.xyz = vPosition.xyz + teapotBasePos.xyz;
	
	
	//Calculate vertex coordinates in eye space...
	vec4 mvVertex = gl_ModelViewMatrix * vPosition;
	
	//Calculate light coordinates in eye space...
	vec4 mvLightPosition = gl_ModelViewMatrix * lightPosition;
	
	//Eye direction should be the negated vertex position in eye space
	vec3 iEyeDir = -mvVertex.xyz;
	
	//Light direction should be the difference vector in between vertex and light position vectors
	//in eye space
	vec3 iLightDir = mvLightPosition.xyz - mvVertex.xyz;
	
	//Do lighting equations here for gouraud shading
	vec3 N = normalize(iNormal);
    vec3 L = normalize(iLightDir);
    
    //COLOR COMPUTATION HERE
    
    iAmbientColor = matAmbientCoef.rgb;
    iDiffuseColor = vec4( 0.0,0.0,0.0,matDiffuseCoef.a );
    iSpecularColor = vec3( 0.0,0.0,0.0 );

    float lambertTerm = dot(N,L);
    if (lambertTerm > 0.0)
    {
		float r = length( lightPosition.xyz - iPos );
		float k = 1 / pow(lightAttRange,2.0);
		float I = length(lightColor);
		
		float lightAtt = clamp( I / (1 + k * pow( r, lightAttExp ) ),0.0,1.0 );
		    
		iDiffuseColor.rgb += (lightColor.rgb * matDiffuseCoef.rgb * lambertTerm ) * lightAtt;
		vec3 E = normalize(iEyeDir);
		vec3 R = reflect(-L, N);
		float specular = pow(max(dot(R, E), 0.0), matShininess);
		iSpecularColor += (lightColor.rgb * matSpecularCoef.rgb * specular) * lightAtt;
    }	
	
	gl_Position = gl_ProjectionMatrix * mvVertex;
	
}